@@ -23,11 +23,12 @@
 
 #include "knot/conf/base.h"
 #include "knot/conf/confdb.h"
+#include "knot/catalog/catalog_db.h"
 #include "knot/common/log.h"
 #include "knot/server/dthreads.h"
-#include "knot/zone/catalog.h"
 #include "libknot/libknot.h"
 #include "libknot/yparser/yptrafo.h"
+#include "libknot/xdp.h"
 #include "contrib/files.h"
 #include "contrib/macros.h"
 #include "contrib/sockaddr.h"
@@ -39,8 +40,8 @@
 
 #define DBG_LOG(err) CONF_LOG(LOG_DEBUG, "%s (%s)", __func__, knot_strerror((err)));
 
-#define DFLT_TCP_WORKERS_MIN		10
-#define DFLT_BG_WORKERS_MAX		10
+#define DFLT_MIN_TCP_WORKERS		10
+#define DFLT_MAX_BG_WORKERS		10
 #define FALLBACK_MAX_TCP_CLIENTS	100
 
 bool conf_db_exists(
@@ -219,8 +220,10 @@ conf_val_t conf_zone_get_txn(
 
 	// Check if this is a catalog member zone.
 	if (conf->catalog != NULL) {
-		knot_dname_storage_t catalog;
-		int ret = catalog_get_zone_threadsafe(conf->catalog, dname, catalog);
+		void *tofree = NULL;
+		const knot_dname_t *catalog;
+		const char *group;
+		int ret = catalog_get_catz(conf->catalog, dname, &catalog, &group, &tofree);
 		if (ret == KNOT_EOK) {
 			conf_db_get(conf, txn, C_ZONE, C_CATALOG_TPL, catalog,
 			            knot_dname_size(catalog), &val);
@@ -228,9 +231,19 @@ conf_val_t conf_zone_get_txn(
 				CONF_LOG_ZONE(LOG_ERR, catalog,
 				              "catalog zone has no catalog template (%s)",
 				              knot_strerror(val.code));
+				free(tofree);
 				return val;
 			}
 			conf_val(&val);
+			while (val.code == KNOT_EOK) {
+				if (strmemcmp(group, val.data, val.len) == 0) {
+					break;
+				}
+				conf_val_next(&val);
+			}
+			conf_val(&val); // Use first value if no match.
+			free(tofree);
+
 			conf_db_get(conf, txn, C_TPL, key1_name, val.data, val.len, &val);
 			goto got_template;
 		}
@@ -1108,29 +1121,13 @@ char* conf_zonefile_txn(
 	return get_filename(conf, txn, zone, file);
 }
 
-inline static bool legacy_db_fallback(
-	const yp_name_t *db_type)
-{
-	return (db_type[1] == C_JOURNAL_DB[1] ||
-	        db_type[1] == C_KASP_DB[1] ||
-	        db_type[1] == C_TIMER_DB[1]);
-	// warning comparing by first letter as every database has different!
-}
-
 char* conf_db_txn(
 	conf_t *conf,
 	knot_db_txn_t *txn,
 	const yp_name_t *db_type)
 {
 	conf_val_t storage_val = conf_get_txn(conf, txn, C_DB, C_STORAGE);
-	if (storage_val.code != KNOT_EOK) {
-		storage_val = conf_default_get_txn(conf, txn, C_STORAGE);
-	}
-
 	conf_val_t db_val = conf_get_txn(conf, txn, C_DB, db_type);
-	if (db_val.code != KNOT_EOK && legacy_db_fallback(db_type)) {
-		db_val = conf_default_get_txn(conf, txn, db_type);
-	}
 
 	char *storage = conf_abs_path(&storage_val, NULL);
 	char *dbdir = conf_abs_path(&db_val, storage);
@@ -1139,44 +1136,15 @@ char* conf_db_txn(
 	return dbdir;
 }
 
-conf_val_t conf_db_param_txn(
-	conf_t *conf,
-	knot_db_txn_t *txn,
-	const yp_name_t *param,
-	const yp_name_t *legacy_param)
-{
-	conf_val_t val = conf_get_txn(conf, txn, C_DB, param);
-	if (val.code != KNOT_EOK && legacy_param != NULL) {
-		val = conf_default_get_txn(conf, txn, legacy_param);
-	}
-
-	return val;
-}
-
-bool conf_tcp_reuseport_txn(
-	conf_t *conf,
-	knot_db_txn_t *txn)
-{
-	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_TCP_REUSEPORT);
-	return conf_bool(&val);
-}
-
-bool conf_socket_affinity_txn(
-	conf_t *conf,
-	knot_db_txn_t *txn)
-{
-	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_SOCKET_AFFINITY);
-	return conf_bool(&val);
-}
-
 size_t conf_udp_threads_txn(
 	conf_t *conf,
 	knot_db_txn_t *txn)
 {
 	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_UDP_WORKERS);
 	int64_t workers = conf_int(&val);
+	assert(workers <= CONF_MAX_UDP_WORKERS);
 	if (workers == YP_NIL) {
-		return dt_optimal_size();
+		return MIN(dt_optimal_size(), CONF_MAX_UDP_WORKERS);
 	}
 
 	return workers;
@@ -1188,8 +1156,10 @@ size_t conf_tcp_threads_txn(
 {
 	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_TCP_WORKERS);
 	int64_t workers = conf_int(&val);
+	assert(workers <= CONF_MAX_TCP_WORKERS);
 	if (workers == YP_NIL) {
-		return MAX(dt_optimal_size(), DFLT_TCP_WORKERS_MIN);
+		size_t optimal = MAX(dt_optimal_size(), DFLT_MIN_TCP_WORKERS);
+		return MIN(optimal, CONF_MAX_TCP_WORKERS);
 	}
 
 	return workers;
@@ -1201,7 +1171,10 @@ size_t conf_xdp_threads_txn(
 {
 	size_t workers = 0;
 
-	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_LISTEN_XDP);
+	conf_val_t val = conf_get_txn(conf, txn, C_XDP, C_LISTEN);
+	if (val.code != KNOT_EOK) {
+		val = conf_get_txn(conf, txn, C_SRV, C_LISTEN_XDP);
+	}
 	while (val.code == KNOT_EOK) {
 		struct sockaddr_storage addr = conf_addr(&val, NULL);
 		conf_xdp_iface_t iface;
@@ -1221,8 +1194,10 @@ size_t conf_bg_threads_txn(
 {
 	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_BG_WORKERS);
 	int64_t workers = conf_int(&val);
+	assert(workers <= CONF_MAX_BG_WORKERS);
 	if (workers == YP_NIL) {
-		return MIN(dt_optimal_size(), DFLT_BG_WORKERS_MAX);
+		assert(DFLT_MAX_BG_WORKERS <= CONF_MAX_BG_WORKERS);
+		return MIN(dt_optimal_size(), DFLT_MAX_BG_WORKERS);
 	}
 
 	return workers;
@@ -1233,9 +1208,6 @@ size_t conf_tcp_max_clients_txn(
 	knot_db_txn_t *txn)
 {
 	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_TCP_MAX_CLIENTS);
-	if (val.code != KNOT_EOK) {
-		val = conf_get_txn(conf, txn, C_SRV, C_MAX_TCP_CLIENTS);
-	}
 	int64_t clients = conf_int(&val);
 	if (clients == YP_NIL) {
 		static size_t permval = 0;
