@@ -647,61 +647,34 @@ static inline char* conf_db(
 /*!
  * Gets database-specific parameter.
  *
- * \param[in] conf          Configuration.
- * \param[in] txn           Configuration DB transaction.
- * \param[in] param         Parameter name.
- * \param[in] legacy_param  Legacy parameter name.
+ * \param[in] conf   Configuration.
+ * \param[in] param  Parameter name.
  *
  * \return Item value.
  */
-conf_val_t conf_db_param_txn(
-	conf_t *conf,
-	knot_db_txn_t *txn,
-	const yp_name_t *param,
-	const yp_name_t *legacy_param
-);
 static inline conf_val_t conf_db_param(
 	conf_t *conf,
-	const yp_name_t *param,
-	const yp_name_t *legacy_param)
+	const yp_name_t *param)
 {
-	return conf_db_param_txn(conf, &conf->read_txn, param, legacy_param);
+	return conf_get_txn(conf, &conf->read_txn, C_DB, param);
 }
 
 /*!
- * Gets the configured setting of the TCP reuseport switch.
+ * Gets the configured setting of the bool option in the specified section.
  *
- * \param[in] conf  Configuration.
- * \param[in] txn   Configuration DB transaction.
+ * \param[in] conf     Configuration.
+ * \param[in] section  Section name.
+ * \param[in] param    Parameter name.
  *
  * \return True if enabled, false otherwise.
  */
-bool conf_tcp_reuseport_txn(
+static inline bool conf_get_bool(
 	conf_t *conf,
-	knot_db_txn_t *txn
-);
-static inline bool conf_tcp_reuseport(
-	conf_t *conf)
+	const yp_name_t *section,
+	const yp_name_t *param)
 {
-	return conf_tcp_reuseport_txn(conf, &conf->read_txn);
-}
-
-/*!
- * Gets the configured setting of the socket affinity switch.
- *
- * \param[in] conf  Configuration.
- * \param[in] txn   Configuration DB transaction.
- *
- * \return True if enabled, false otherwise.
- */
-bool conf_socket_affinity_txn(
-	conf_t *conf,
-	knot_db_txn_t *txn
-);
-static inline bool conf_socket_affinity(
-	conf_t *conf)
-{
-	return conf_socket_affinity_txn(conf, &conf->read_txn);
+	conf_val_t val = conf_get_txn(conf, &conf->read_txn, section, param);
+	return conf_bool(&val);
 }
 
 /*!
@@ -776,6 +749,28 @@ static inline size_t conf_bg_threads(
 	return conf_bg_threads_txn(conf, &conf->read_txn);
 }
 
+/*!
+ * Gets the required LMDB readers limit based on the current configuration.
+ *
+ * \note The resulting value is a common limit to journal, kasp, timers,
+ *       and catalog databases. So it's over-estimated for simpicity reasons.
+ *
+ * \note This function cannot be used for the configuration database setting :-/
+ *
+ * \param[in] conf  Configuration.
+ *
+ * \return Number of readers.
+ */
+static inline size_t conf_lmdb_readers(
+	conf_t *conf)
+{
+	if (conf == NULL) { // Return default in tests.
+		return 126;
+	}
+	return conf_udp_threads(conf) + conf_tcp_threads(conf) +
+	       conf_bg_threads(conf) + conf_xdp_threads(conf);
+}
+
 /*!
  * Gets the configured maximum number of TCP clients.
  *
