@@ -30,12 +30,13 @@
 #define C_ASYNC_START		"\x0B""async-start"
 #define C_BACKEND		"\x07""backend"
 #define C_BG_WORKERS		"\x12""background-workers"
-#define C_BLOCK_NOTIFY_START	"\x18""block-notify-after-start"
 #define C_BLOCK_NOTIFY_XFR	"\x1B""block-notify-after-transfer"
-#define C_CATALOG_DB            "\x0A""catalog-db"
-#define C_CATALOG_DB_MAX_SIZE   "\x13""catalog-db-max-size"
+#define C_CATALOG_DB		"\x0A""catalog-db"
+#define C_CATALOG_DB_MAX_SIZE	"\x13""catalog-db-max-size"
+#define C_CATALOG_GROUP		"\x0D""catalog-group"
 #define C_CATALOG_ROLE		"\x0C""catalog-role"
 #define C_CATALOG_TPL		"\x10""catalog-template"
+#define C_CATALOG_ZONE		"\x0C""catalog-zone"
 #define C_CDS_CDNSKEY		"\x13""cds-cdnskey-publish"
 #define C_CDS_DIGESTTYPE	"\x0F""cds-digest-type"
 #define C_CHK_INTERVAL		"\x0E""check-interval"
@@ -45,7 +46,6 @@
 #define C_DB			"\x08""database"
 #define C_DDNS_MASTER		"\x0B""ddns-master"
 #define C_DENY			"\x04""deny"
-#define C_DISABLE_ANY		"\x0B""disable-any"
 #define C_DNSKEY_TTL		"\x0A""dnskey-ttl"
 #define C_DNSSEC_POLICY		"\x0D""dnssec-policy"
 #define C_DNSSEC_SIGNING	"\x0E""dnssec-signing"
@@ -73,7 +73,6 @@
 #define C_KSK_SHARED		"\x0a""ksk-shared"
 #define C_KSK_SIZE		"\x08""ksk-size"
 #define C_LISTEN		"\x06""listen"
-#define C_LISTEN_XDP		"\x0A""listen-xdp"
 #define C_LOG			"\x03""log"
 #define C_MANUAL		"\x06""manual"
 #define C_MASTER		"\x06""master"
@@ -95,6 +94,7 @@
 #define C_REFRESH_MIN_INTERVAL	"\x14""refresh-min-interval"
 #define C_REPRO_SIGNING		"\x14""reproducible-signing"
 #define C_RMT			"\x06""remote"
+#define C_ROUTE_CHECK		"\x0B""route-check"
 #define C_RRSIG_LIFETIME	"\x0E""rrsig-lifetime"
 #define C_RRSIG_PREREFRESH	"\x11""rrsig-pre-refresh"
 #define C_RRSIG_REFRESH		"\x0D""rrsig-refresh"
@@ -111,7 +111,12 @@
 #define C_STATS			"\x0A""statistics"
 #define C_STORAGE		"\x07""storage"
 #define C_TARGET		"\x06""target"
+#define C_TCP			"\x03""tcp"
+#define C_TCP_FASTOPEN		"\x0C""tcp-fastopen"
+#define C_TCP_IDLE_CLOSE	"\x16""tcp-idle-close-timeout"
+#define C_TCP_IDLE_RESET	"\x16""tcp-idle-reset-timeout"
 #define C_TCP_IDLE_TIMEOUT	"\x10""tcp-idle-timeout"
+#define C_TCP_INBUF_MAX_SIZE	"\x12""tcp-inbuf-max-size"
 #define C_TCP_IO_TIMEOUT	"\x0E""tcp-io-timeout"
 #define C_TCP_MAX_CLIENTS	"\x0F""tcp-max-clients"
 #define C_TCP_REUSEPORT		"\x0D""tcp-reuseport"
@@ -134,15 +139,20 @@
 #define C_USER			"\x04""user"
 #define C_VERSION		"\x07""version"
 #define C_VIA			"\x03""via"
+#define C_XDP			"\x03""xdp"
 #define C_ZONE			"\x04""zone"
 #define C_ZONEFILE_LOAD		"\x0D""zonefile-load"
 #define C_ZONEFILE_SYNC		"\x0D""zonefile-sync"
+#define C_ZONEMD_GENERATE	"\x0F""zonemd-generate"
+#define C_ZONEMD_VERIFY		"\x0D""zonemd-verify"
 #define C_ZONE_MAX_SIZE		"\x0D""zone-max-size"
 #define C_ZONE_MAX_TLL		"\x0C""zone-max-ttl"
 #define C_ZSK_LIFETIME		"\x0C""zsk-lifetime"
 #define C_ZSK_SIZE		"\x08""zsk-size"
 
 // Legacy items.
+#define C_DISABLE_ANY		"\x0B""disable-any"
+#define C_LISTEN_XDP		"\x0A""listen-xdp"
 #define C_MAX_TIMER_DB_SIZE	"\x11""max-timer-db-size"
 #define C_MAX_JOURNAL_DB_SIZE	"\x13""max-journal-db-size"
 #define C_MAX_KASP_DB_SIZE	"\x10""max-kasp-db-size"
@@ -185,6 +195,12 @@ enum {
 	SERIAL_POLICY_DATESERIAL = 3,
 };
 
+enum {
+	ZONE_DIGEST_NONE   = 0,
+	ZONE_DIGEST_SHA384 = 1,
+	ZONE_DIGEST_SHA512 = 2,
+};
+
 enum {
 	JOURNAL_CONTENT_NONE    = 0,
 	JOURNAL_CONTENT_CHANGES = 1,
@@ -206,6 +222,8 @@ enum {
 enum {
 	CATALOG_ROLE_NONE      = 0,
 	CATALOG_ROLE_INTERPRET = 1,
+	CATALOG_ROLE_GENERATE  = 2,
+	CATALOG_ROLE_MEMBER    = 3,
 };
 
 extern const knot_lookup_t acl_actions[];
