@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,11 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#ifdef ENABLE_XDP
+#include <linux/ip.h>
+#include <linux/ipv6.h>
+#include <linux/udp.h>
+#endif
 
 #include "libdnssec/key.h"
 #include "knot/conf/tools.h"
@@ -33,6 +38,7 @@
 #include "knot/common/log.h"
 #include "libknot/errcode.h"
 #include "libknot/yparser/yptrafo.h"
+#include "libknot/xdp.h"
 #include "contrib/sockaddr.h"
 #include "contrib/string.h"
 #include "contrib/wire_ctx.h"
@@ -241,7 +247,16 @@ int check_listen(
 	return KNOT_EOK;
 }
 
-int check_xdp(
+int check_xdp_listen_old(
+	knotd_conf_check_args_t *args)
+{
+	CONF_LOG(LOG_NOTICE, "option 'server.listen-xdp' is obsolete, "
+	                     "use option 'xdp.listen' instead");
+
+	return KNOT_EOK;
+}
+
+int check_xdp_listen(
 	knotd_conf_check_args_t *args)
 {
 #ifndef ENABLE_XDP
@@ -257,8 +272,8 @@ int check_xdp(
 		return ret;
 	}
 
-	conf_val_t xdp = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
-	                              C_LISTEN_XDP);
+	conf_val_t xdp = conf_get_txn(args->extra->conf, args->extra->txn, C_XDP,
+	                              C_LISTEN);
 	size_t count = conf_val_count(&xdp);
 	while (xdp.code == KNOT_EOK && count-- > 1) {
 		struct sockaddr_storage addr = conf_addr(&xdp, NULL);
@@ -330,7 +345,7 @@ int check_module_id(
 	conf_val_t val = conf_get_txn(args->extra->conf, args->extra->txn, \
 	                              section, old_item); \
 	if (val.code == KNOT_EOK) { \
-		CONF_LOG(LOG_NOTICE, "option '%s.%s' is obsolete, " \
+		CONF_LOG(LOG_NOTICE, "option '%s.%s' has no effect, " \
 		                     "use option '%s.%s' instead", \
 		                     &section[1], &old_item[1], \
 		                     &section[1], &new_item[1]); \
@@ -341,30 +356,66 @@ int check_module_id(
 	conf_val_t val = conf_rawid_get_txn(args->extra->conf, args->extra->txn, \
 	                                    section, old_item, args->id, args->id_len); \
 	if (val.code == KNOT_EOK) { \
-		CONF_LOG(LOG_NOTICE, "option '%s.%s' is obsolete, " \
+		CONF_LOG(LOG_NOTICE, "option '%s.%s' has no effect, " \
 		                     "use option '%s.%s' instead", \
 		                     &section[1], &old_item[1], \
 		                     &section[1], &new_item[1]); \
 	} \
 }
 
+static void check_mtu(knotd_conf_check_args_t *args, conf_val_t *xdp_listen)
+{
+#ifdef ENABLE_XDP
+	conf_val_t val = conf_get_txn(args->extra->conf, args->extra->txn,
+	                              C_SRV, C_UDP_MAX_PAYLOAD_IPV4);
+	if (val.code != KNOT_EOK) {
+		val = conf_get_txn(args->extra->conf, args->extra->txn,
+		                   C_SRV, C_UDP_MAX_PAYLOAD);
+	}
+	int64_t ipv4_max = conf_int(&val) + sizeof(struct udphdr) + 4 + // Eth. CRC
+	                   sizeof(struct iphdr) + sizeof(struct ethhdr);
+
+	val = conf_get_txn(args->extra->conf, args->extra->txn,
+	                   C_SRV, C_UDP_MAX_PAYLOAD_IPV6);
+	if (val.code != KNOT_EOK) {
+		val = conf_get_txn(args->extra->conf, args->extra->txn,
+		                   C_SRV, C_UDP_MAX_PAYLOAD);
+	}
+	int64_t ipv6_max = conf_int(&val) + sizeof(struct udphdr) + 4 + // Eth. CRC
+	                   sizeof(struct ipv6hdr) + sizeof(struct ethhdr);
+
+	if (ipv6_max > KNOT_XDP_MAX_MTU || ipv4_max > KNOT_XDP_MAX_MTU) {
+		CONF_LOG(LOG_WARNING, "maximum UDP payload not compatible with XDP MTU (%u)",
+		         KNOT_XDP_MAX_MTU);
+	}
+
+	while (xdp_listen->code == KNOT_EOK) {
+		struct sockaddr_storage addr = conf_addr(xdp_listen, NULL);
+		conf_xdp_iface_t iface;
+		int ret = conf_xdp_iface(&addr, &iface);
+		if (ret != KNOT_EOK) {
+			CONF_LOG(LOG_WARNING, "failed to check XDP interface MTU");
+			return;
+		}
+		int mtu = knot_eth_mtu(iface.name);
+		if (mtu < 0) {
+			CONF_LOG(LOG_WARNING, "failed to read MTU of interface %s",
+			         iface.name);
+			continue;
+		}
+		mtu += sizeof(struct ethhdr) + 4;
+		if (ipv6_max > mtu || ipv4_max > mtu) {
+			CONF_LOG(LOG_WARNING, "maximum UDP payload not compatible "
+			                      "with MTU of interface %s", iface.name);
+		}
+		conf_val_next(xdp_listen);
+	}
+#endif
+}
+
 int check_server(
 	knotd_conf_check_args_t *args)
 {
-	conf_val_t listen = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
-	                                 C_LISTEN);
-	conf_val_t xdp = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
-	                              C_LISTEN_XDP);
-	if (xdp.code == KNOT_EOK && listen.code != KNOT_EOK) {
-		CONF_LOG(LOG_WARNING, "unable to process TCP queries due to XDP-only interfaces");
-	}
-
-	conf_val_t hshake = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
-	                                 C_TCP_HSHAKE_TIMEOUT);
-	if (hshake.code == KNOT_EOK) {
-		CONF_LOG(LOG_NOTICE, "option 'server.tcp-handshake-timeout' is no longer supported");
-	}
-
 	CHECK_LEGACY_NAME(C_SRV, C_TCP_REPLY_TIMEOUT, C_TCP_RMT_IO_TIMEOUT);
 	CHECK_LEGACY_NAME(C_SRV, C_MAX_TCP_CLIENTS, C_TCP_MAX_CLIENTS);
 	CHECK_LEGACY_NAME(C_SRV, C_MAX_UDP_PAYLOAD, C_UDP_MAX_PAYLOAD);
@@ -374,6 +425,25 @@ int check_server(
 	return KNOT_EOK;
 }
 
+int check_xdp(
+	knotd_conf_check_args_t *args)
+{
+	conf_val_t xdp_listen = conf_get_txn(args->extra->conf, args->extra->txn,
+	                                     C_XDP, C_LISTEN);
+	conf_val_t srv_listen = conf_get_txn(args->extra->conf, args->extra->txn,
+	                                     C_SRV, C_LISTEN);
+	conf_val_t tcp = conf_get_txn(args->extra->conf, args->extra->txn, C_XDP,
+	                              C_TCP);
+	if (xdp_listen.code == KNOT_EOK) {
+		if (srv_listen.code != KNOT_EOK && tcp.code != KNOT_EOK) {
+			CONF_LOG(LOG_WARNING, "TCP processing not available");
+		}
+		check_mtu(args, &xdp_listen);
+	}
+
+	return KNOT_EOK;
+}
+
 int check_keystore(
 	knotd_conf_check_args_t *args)
 {
@@ -568,7 +638,7 @@ int check_remote(
 	conf_val_t val = conf_rawid_get_txn(args->extra->conf, args->extra->txn, \
 	                                    C_TPL, old_item, args->id, args->id_len); \
 	if (val.code == KNOT_EOK) { \
-		CONF_LOG(LOG_NOTICE, "option 'template.%s' is obsolete, " \
+		CONF_LOG(LOG_NOTICE, "option 'template.%s' has no effect, " \
 		                     "use option 'database.%s' instead", \
 		                     &old_item[1], &new_item[1]); \
 	} \
@@ -583,6 +653,18 @@ int check_remote(
 	} \
 }
 
+int check_catalog_group(
+	knotd_conf_check_args_t *args)
+{
+	assert(args->data_len > 0);
+	if (args->data_len - 1 > 255) {
+		args->err_str = "group name longer than 255 characters";
+		return KNOT_EINVAL;
+	}
+
+	return KNOT_EOK;
+}
+
 int check_template(
 	knotd_conf_check_args_t *args)
 {
@@ -600,24 +682,7 @@ int check_template(
 	CHECK_LEGACY_NAME_ID(C_TPL, C_MAX_JOURNAL_DEPTH, C_JOURNAL_MAX_DEPTH);
 	CHECK_LEGACY_NAME_ID(C_TPL, C_MAX_JOURNAL_USAGE, C_JOURNAL_MAX_USAGE);
 
-	conf_val_t any = conf_rawid_get_txn(args->extra->conf, args->extra->txn,
-	                                    C_TPL, C_DISABLE_ANY, args->id, args->id_len);
-	if (any.code == KNOT_EOK) {
-		CONF_LOG(LOG_NOTICE, "option 'disable-any' is deprecated and has no effect");
-	}
-
-	if (is_default_id(args->id, args->id_len)) {
-		conf_val_t db_storage = conf_get_txn(args->extra->conf, args->extra->txn,
-		                                     C_DB, C_STORAGE);
-		conf_val_t tpl_storage = conf_rawid_get_txn(args->extra->conf, args->extra->txn,
-		                                            C_TPL, C_STORAGE, args->id, args->id_len);
-		if (db_storage.code != KNOT_EOK && tpl_storage.code == KNOT_EOK &&
-		    strcmp(conf_str(&tpl_storage), STORAGE_DIR) != 0) {
-			CONF_LOG(LOG_NOTICE, "non-default 'template[default].storage' detected, "
-			                     "please configure also 'db.storage' to avoid compatibility "
-			                     "issues with future versions");
-		}
-	} else {
+	if (!is_default_id(args->id, args->id_len)) {
 		CHECK_DFLT(C_GLOBAL_MODULE, "global module");
 	}
 
@@ -633,20 +698,14 @@ int check_zone(
 	CHECK_LEGACY_NAME_ID(C_ZONE, C_MAX_JOURNAL_DEPTH, C_JOURNAL_MAX_DEPTH);
 	CHECK_LEGACY_NAME_ID(C_ZONE, C_MAX_JOURNAL_USAGE, C_JOURNAL_MAX_USAGE);
 
-	conf_val_t any = conf_rawid_get_txn(args->extra->conf, args->extra->txn,
-	                                    C_TPL, C_DISABLE_ANY, args->id, args->id_len);
-	if (any.code == KNOT_EOK) {
-		CONF_LOG(LOG_NOTICE, "option 'disable-any' is deprecated and has no effect");
-	}
-
 	conf_val_t zf_load = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                       C_ZONEFILE_LOAD, yp_dname(args->id));
 	conf_val_t journal = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                       C_JOURNAL_CONTENT, yp_dname(args->id));
 	if (conf_opt(&zf_load) == ZONEFILE_LOAD_DIFSE &&
 	    conf_opt(&journal) != JOURNAL_CONTENT_ALL) {
-		CONF_LOG(LOG_NOTICE, "'zonefile-load: difference-no-serial' will require 'journal-content: all' "
-		                     "in versions >= 3.1");
+		args->err_str = "'zonefile-load: difference-no-serial' requires 'journal-content: all'";
+		return KNOT_EINVAL;
 	}
 
 	conf_val_t signing = conf_zone_get_txn(args->extra->conf, args->extra->txn,
@@ -662,11 +721,18 @@ int check_zone(
 	                                            C_CATALOG_ROLE, yp_dname(args->id));
 	conf_val_t catalog_tpl = conf_zone_get_txn(args->extra->conf, args->extra->txn,
 	                                           C_CATALOG_TPL, yp_dname(args->id));
+	conf_val_t catalog_zone = conf_zone_get_txn(args->extra->conf, args->extra->txn,
+	                                            C_CATALOG_ZONE, yp_dname(args->id));
 	if ((bool)(conf_opt(&catalog_role) == CATALOG_ROLE_INTERPRET) !=
 	    (bool)(catalog_tpl.code == KNOT_EOK)) {
 		args->err_str = "'catalog-role' must correspond to configured 'catalog-template'";
 		return KNOT_EINVAL;
 	}
+	if ((bool)(conf_opt(&catalog_role) == CATALOG_ROLE_MEMBER) !=
+	    (bool)(catalog_zone.code == KNOT_EOK)) {
+		args->err_str = "'catalog-role' must correspond to configured 'catalog-zone'";
+		return KNOT_EINVAL;
+	}
 
 	return KNOT_EOK;
 }
