@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,12 +33,24 @@
 #define CTL_FILTER_STATUS_EVENTS	'e'
 
 #define CTL_FILTER_PURGE_EXPIRE		'e'
-#define CTL_FILTER_PURGE_TIMERS		't'
 #define CTL_FILTER_PURGE_ZONEFILE	'f'
 #define CTL_FILTER_PURGE_JOURNAL	'j'
+#define CTL_FILTER_PURGE_TIMERS		't'
 #define CTL_FILTER_PURGE_KASPDB		'k'
 #define CTL_FILTER_PURGE_ORPHAN		'o'
 
+#define CTL_FILTER_BACKUP_OUTDIR	'd'
+#define CTL_FILTER_BACKUP_ZONEFILE	'z'
+#define CTL_FILTER_BACKUP_NOZONEFILE	'Z'
+#define CTL_FILTER_BACKUP_JOURNAL	'j'
+#define CTL_FILTER_BACKUP_NOJOURNAL	'J'
+#define CTL_FILTER_BACKUP_TIMERS	't'
+#define CTL_FILTER_BACKUP_NOTIMERS	'T'
+#define CTL_FILTER_BACKUP_KASPDB	'k'
+#define CTL_FILTER_BACKUP_NOKASPDB	'K'
+#define CTL_FILTER_BACKUP_CATALOG	'c'
+#define CTL_FILTER_BACKUP_NOCATALOG	'C'
+
 /*! Control commands. */
 typedef enum {
 	CTL_NONE,
@@ -91,8 +103,7 @@ typedef struct {
 	knot_ctl_type_t type;
 	knot_ctl_data_t data;
 	server_t *server;
-	bool failed;
-	void *custom_ctx;
+	bool suppress;	// Suppress error reporting in the "all zones" ctl commands.
 } ctl_args_t;
 
 /*!
