@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -244,9 +244,14 @@ static void event_wrap(task_t *task)
 
 	if (blocking != NULL) {
 		events->blocking[type] = NULL;
+		events->result[type] = ret;
 		pthread_cond_broadcast(blocking);
 	}
 
+	if (events->run_end != NULL) {
+		pthread_cond_broadcast(events->run_end);
+	}
+
 	pthread_mutex_unlock(&events->mx);
 	reschedule(events);
 }
@@ -370,10 +375,10 @@ void zone_events_schedule_user(zone_t *zone, zone_event_type_t type)
 	reschedule(events);
 }
 
-void zone_events_schedule_blocking(zone_t *zone, zone_event_type_t type, bool user)
+int zone_events_schedule_blocking(zone_t *zone, zone_event_type_t type, bool user)
 {
 	if (!zone || !valid_event(type)) {
-		return;
+		return KNOT_EINVAL;
 	}
 
 	zone_events_t *events = &zone->events;
@@ -397,8 +402,11 @@ void zone_events_schedule_blocking(zone_t *zone, zone_event_type_t type, bool us
 	while (events->blocking[type] == &local_cond) {
 		pthread_cond_wait(&local_cond, &events->mx);
 	}
+	int ret = events->result[type];
 	pthread_mutex_unlock(&events->mx);
 	pthread_cond_destroy(&local_cond);
+
+	return ret;
 }
 
 void zone_events_enqueue(zone_t *zone, zone_event_type_t type)
@@ -445,6 +453,29 @@ void zone_events_freeze(zone_t *zone)
 	evsched_cancel(events->event);
 }
 
+void zone_events_freeze_blocking(zone_t *zone)
+{
+	if (!zone) {
+		return;
+	}
+
+	zone_events_freeze(zone);
+
+	zone_events_t *events = &zone->events;
+
+	/* Wait for running event to finish. */
+	pthread_cond_t cond;
+	pthread_cond_init(&cond, NULL);
+	pthread_mutex_lock(&events->mx);
+	while (events->running) {
+		events->run_end = &cond;
+		pthread_cond_wait(&cond, &events->mx);
+	}
+	events->run_end = NULL;
+	pthread_mutex_unlock(&events->mx);
+	pthread_cond_destroy(&cond);
+}
+
 void zone_events_start(zone_t *zone)
 {
 	if (!zone) {
