@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 #include <assert.h>
 
+#include "knot/catalog/generate.h"
 #include "knot/common/log.h"
 #include "knot/conf/conf.h"
 #include "knot/dnssec/key-events.h"
@@ -66,7 +67,9 @@ int event_load(conf_t *conf, zone_t *zone)
 	int ret = KNOT_EOK;
 
 	// If configured, load journal contents.
-	if (load_from == JOURNAL_CONTENT_ALL && !old_contents_exist && zf_from != ZONEFILE_LOAD_WHOLE) {
+	if (!old_contents_exist &&
+	    ((load_from == JOURNAL_CONTENT_ALL && zf_from != ZONEFILE_LOAD_WHOLE) ||
+	     zone->cat_members != NULL)) {
 		ret = zone_load_from_journal(conf, zone, &journal_conts);
 		switch (ret) {
 		case KNOT_EOK:
@@ -83,7 +86,7 @@ int event_load(conf_t *conf, zone_t *zone)
 	}
 
 	// If configured, attempt to load zonefile.
-	if (zf_from != ZONEFILE_LOAD_NONE) {
+	if (zf_from != ZONEFILE_LOAD_NONE && zone->cat_members == NULL) {
 		struct timespec mtime;
 		char *filename = conf_zonefile(conf, zone->name);
 		ret = zonefile_exists(filename, &mtime);
@@ -116,7 +119,7 @@ int event_load(conf_t *conf, zone_t *zone)
 		if (zf_conts != NULL && zf_from == ZONEFILE_LOAD_DIFSE && relevant != NULL) {
 			uint32_t serial = zone_contents_serial(relevant);
 			conf_val_t policy = conf_zone_get(conf, C_SERIAL_POLICY, zone->name);
-			uint32_t set = serial_next(serial, conf_opt(&policy));
+			uint32_t set = serial_next(serial, conf_opt(&policy), 1);
 			zone_contents_set_soa_serial(zf_conts, set);
 			log_zone_info(zone->name, "zone file parsed, serial corrected %u -> %u",
 			              zone->zonefile.serial, set);
@@ -141,10 +144,19 @@ int event_load(conf_t *conf, zone_t *zone)
 			}
 		}
 	}
+	if (zone->cat_members != NULL && !old_contents_exist) {
+		uint32_t serial = journal_conts == NULL ? 1 : zone_contents_serial(journal_conts);
+		serial = serial_next(serial, SERIAL_POLICY_UNIXTIME, 1); // unixtime hardcoded
+		zf_conts = catalog_update_to_zone(zone->cat_members, zone->name, serial);
+		if (zf_conts == NULL) {
+			ret = zone->cat_members->error == KNOT_EOK ? KNOT_ENOMEM : zone->cat_members->error;
+			goto cleanup;
+		}
+	}
 
 	// If configured contents=all, but not present, store zonefile.
-	if (load_from == JOURNAL_CONTENT_ALL && !zone_in_journal_exists &&
-	    (zf_conts != NULL || old_contents_exist)) {
+	if ((load_from == JOURNAL_CONTENT_ALL || zone->cat_members != NULL) &&
+	    !zone_in_journal_exists && (zf_conts != NULL || old_contents_exist)) {
 		zone_contents_t *store_c = old_contents_exist ? zone->contents : zf_conts;
 		ret = zone_in_journal_store(conf, zone, store_c);
 		if (ret != KNOT_EOK) {
@@ -156,13 +168,21 @@ int event_load(conf_t *conf, zone_t *zone)
 	}
 
 	val = conf_zone_get(conf, C_DNSSEC_SIGNING, zone->name);
-	bool dnssec_enable = conf_bool(&val), zu_from_zf_conts = false;
-	bool do_diff = (zf_from == ZONEFILE_LOAD_DIFF || zf_from == ZONEFILE_LOAD_DIFSE);
+	bool dnssec_enable = (conf_bool(&val) && zone->cat_members == NULL), zu_from_zf_conts = false;
+	bool do_diff = (zf_from == ZONEFILE_LOAD_DIFF || zf_from == ZONEFILE_LOAD_DIFSE || zone->cat_members != NULL);
 	bool ignore_dnssec = (do_diff && dnssec_enable);
 
 	// Create zone_update structure according to current state.
 	if (old_contents_exist) {
-		if (zf_conts == NULL) {
+		if (zone->cat_members != NULL) {
+			ret = zone_update_init(&up, zone, UPDATE_INCREMENTAL);
+			if (ret == KNOT_EOK) {
+				ret = catalog_update_to_update(zone->cat_members, &up);
+			}
+			if (ret == KNOT_EOK) {
+				ret = zone_update_increment_soa(&up, conf);
+			}
+		} else if (zf_conts == NULL) {
 			// nothing to be re-loaded
 			ret = KNOT_EOK;
 			goto cleanup;
@@ -174,16 +194,16 @@ int event_load(conf_t *conf, zone_t *zone)
 			zu_from_zf_conts = true;
 		} else {
 			// compute ZF diff and if success, apply it
-			ret = zone_update_from_differences(&up, zone, NULL, zf_conts, UPDATE_INCREMENTAL, ignore_dnssec);
+			ret = zone_update_from_differences(&up, conf, zone, NULL, zf_conts, UPDATE_INCREMENTAL, ignore_dnssec);
 		}
 	} else {
-		if (journal_conts != NULL && zf_from != ZONEFILE_LOAD_WHOLE) {
+		if (journal_conts != NULL && (zf_from != ZONEFILE_LOAD_WHOLE || zone->cat_members != NULL)) {
 			if (zf_conts == NULL) {
 				// load zone-in-journal
 				ret = zone_update_from_contents(&up, zone, journal_conts, UPDATE_HYBRID);
 			} else {
 				// load zone-in-journal, compute ZF diff and if success, apply it
-				ret = zone_update_from_differences(&up, zone, journal_conts, zf_conts,
+				ret = zone_update_from_differences(&up, conf, zone, journal_conts, zf_conts,
 				                                   UPDATE_HYBRID, ignore_dnssec);
 				if (ret == KNOT_ESEMCHECK || ret == KNOT_ERANGE) {
 					log_zone_warning(zone->name,
@@ -228,12 +248,17 @@ int event_load(conf_t *conf, zone_t *zone)
 		goto cleanup;
 	}
 
+	ret = zone_update_verify_digest(conf, &up);
+	if (ret != KNOT_EOK) {
+		goto cleanup;
+	}
+
 	uint32_t middle_serial = zone_contents_serial(up.new_cont);
 
 	if (do_diff && old_contents_exist && dnssec_enable && zf_conts != NULL &&
 	    zone_contents_serial(zf_conts) != zone_contents_serial(zone->contents) &&
 	    !zone_in_journal_exists) {
-		ret = zone_update_start_extra(&up);
+		ret = zone_update_start_extra(&up, conf);
 		if (ret != KNOT_EOK) {
 			goto cleanup;
 		}
@@ -246,7 +271,7 @@ int event_load(conf_t *conf, zone_t *zone)
 	// Sign zone using DNSSEC if configured.
 	zone_sign_reschedule_t dnssec_refresh = { 0 };
 	if (dnssec_enable) {
-		ret = knot_dnssec_zone_sign(&up, 0, KEY_ROLL_ALLOW_ALL, 0, &dnssec_refresh);
+		ret = knot_dnssec_zone_sign(&up, conf, 0, KEY_ROLL_ALLOW_ALL, 0, &dnssec_refresh);
 		if (ret != KNOT_EOK) {
 			goto cleanup;
 		}
@@ -259,7 +284,8 @@ int event_load(conf_t *conf, zone_t *zone)
 	}
 
 	// If the change is only automatically incremented SOA serial, make it no change.
-	if (zf_from == ZONEFILE_LOAD_DIFSE && (up.flags & (UPDATE_INCREMENTAL | UPDATE_HYBRID)) &&
+	if ((zf_from == ZONEFILE_LOAD_DIFSE || zone->cat_members != NULL) &&
+	    (up.flags & (UPDATE_INCREMENTAL | UPDATE_HYBRID)) &&
 	    changeset_differs_just_serial(&up.change)) {
 		changeset_t *cpy = changeset_clone(&up.change);
 		if (cpy == NULL) {
@@ -292,6 +318,10 @@ int event_load(conf_t *conf, zone_t *zone)
 	log_zone_info(zone->name, "loaded, serial %s -> %u%s, %zu bytes",
 	              old_serial_str, middle_serial, new_serial_str, zone->contents->size);
 
+	if (zone->cat_members != NULL) {
+		catalog_update_clear(zone->cat_members);
+	}
+
 	// Schedule depedent events.
 	const knot_rdataset_t *soa = zone_soa(zone);
 	zone->timers.soa_expire = knot_soa_expire(soa->rdata);
@@ -301,11 +331,8 @@ int event_load(conf_t *conf, zone_t *zone)
 	}
 
 	replan_from_timers(conf, zone);
-	
-	val = conf_get(conf, C_SRV, C_BLOCK_NOTIFY_START);
-	if ((old_contents_exist && old_serial != new_serial) ||
-	    (!old_contents_exist && !conf_bool(&val)))
-	{
+
+	if (!zone_timers_serial_notified(&zone->timers, new_serial)) {
 		zone_events_schedule_now(zone, ZONE_EVENT_NOTIFY);
 	}
 
