@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -145,6 +145,9 @@ static int process_normal(conf_t *conf, zone_t *zone, list_t *requests)
 
 	// Process all updates.
 	ret = process_bulk(zone, requests, &up);
+	if (ret == KNOT_EOK) {
+		ret = zone_update_verify_digest(conf, &up);
+	}
 	if (ret != KNOT_EOK) {
 		zone_update_clear(&up);
 		set_rcodes(requests, KNOT_RCODE_SERVFAIL);
@@ -156,7 +159,7 @@ static int process_normal(conf_t *conf, zone_t *zone, list_t *requests)
 	bool dnssec_enable = (up.flags & UPDATE_SIGN) && conf_bool(&val);
 	if (dnssec_enable) {
 		zone_sign_reschedule_t resch = { 0 };
-		ret = knot_dnssec_sign_update(&up, &resch);
+		ret = knot_dnssec_sign_update(&up, conf, &resch);
 		if (ret != KNOT_EOK) {
 			zone_update_clear(&up);
 			set_rcodes(requests, KNOT_RCODE_SERVFAIL);
@@ -241,7 +244,8 @@ static int remote_forward(conf_t *conf, knot_request_t *request, conf_remote_t *
 	/* Create a request. */
 	const struct sockaddr_storage *dst = &remote->addr;
 	const struct sockaddr_storage *src = &remote->via;
-	knot_request_t *req = knot_request_make(re.mm, dst, src, query, NULL, 0);
+	knot_request_flag_t flags = conf->cache.srv_tcp_fastopen ? KNOT_REQUEST_TFO : 0;
+	knot_request_t *req = knot_request_make(re.mm, dst, src, query, NULL, flags);
 	if (req == NULL) {
 		knot_requestor_clear(&re);
 		knot_pkt_free(query);
@@ -323,7 +327,7 @@ static void send_update_response(conf_t *conf, const zone_t *zone, knot_request_
 
 		if (net_is_stream(req->fd)) {
 			net_dns_tcp_send(req->fd, req->resp->wire, req->resp->size,
-			                 conf->cache.srv_tcp_remote_io_timeout);
+			                 conf->cache.srv_tcp_remote_io_timeout, NULL);
 		} else {
 			net_dgram_send(req->fd, req->resp->wire, req->resp->size,
 			               &req->remote);
