@@ -492,7 +492,7 @@ static knotd_state_t update_counters(knotd_state_t state, knot_pkt_t *pkt,
 	// Count the request protocol.
 	if (stats->protocol) {
 		bool xdp = qdata->params->xdp_msg != NULL;
-		if (qdata->params->remote->ss_family == AF_INET) {
+		if (knotd_qdata_remote_addr(qdata)->ss_family == AF_INET) {
 			if (qdata->params->flags & KNOTD_QUERY_FLAG_LIMIT_SIZE) {
 				if (xdp) {
 					knotd_mod_stats_incr(mod, tid, CTR_PROTOCOL,
