@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "knot/server/server.h"
 #include "knot/server/tcp-handler.h"
 #include "knot/common/log.h"
+#include "knot/common/fdset.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/query/layer.h"
 #include "contrib/macros.h"
@@ -68,10 +69,11 @@ static void update_sweep_timer(struct timespec *timer)
 static void update_tcp_conf(tcp_context_t *tcp)
 {
 	rcu_read_lock();
+	conf_t *pconf = conf();
 	tcp->max_worker_fds = tcp->client_threshold + \
-		MAX(conf()->cache.srv_tcp_max_clients / conf()->cache.srv_tcp_threads, 1);
-	tcp->idle_timeout = conf()->cache.srv_tcp_idle_timeout;
-	tcp->io_timeout = conf()->cache.srv_tcp_io_timeout;
+		MAX(pconf->cache.srv_tcp_max_clients / pconf->cache.srv_tcp_threads, 1);
+	tcp->idle_timeout = pconf->cache.srv_tcp_idle_timeout;
+	tcp->io_timeout = pconf->cache.srv_tcp_io_timeout;
 	rcu_read_unlock();
 }
 
@@ -85,11 +87,9 @@ static void client_addr(const struct sockaddr_storage *ss, char *out, size_t out
 }
 
 /*! \brief Sweep TCP connection. */
-static enum fdset_sweep_state tcp_sweep(fdset_t *set, int i, void *data)
+static fdset_sweep_state_t tcp_sweep(fdset_t *set, int fd, _unused_ void *data)
 {
-	UNUSED(data);
-	assert(set && i < set->n && i >= 0);
-	int fd = set->pfd[i].fd;
+	assert(set && fd >= 0);
 
 	/* Best-effort, name and shame. */
 	struct sockaddr_storage ss = { 0 };
@@ -100,8 +100,6 @@ static enum fdset_sweep_state tcp_sweep(fdset_t *set, int i, void *data)
 		log_notice("TCP, terminated inactive client, address %s", addr_str);
 	}
 
-	close(fd);
-
 	return FDSET_SWEEP;
 }
 
@@ -133,7 +131,6 @@ static unsigned tcp_set_ifaces(const iface_t *ifaces, size_t n_ifaces,
 		return 0;
 	}
 
-	fdset_clear(fds);
 	for (const iface_t *i = ifaces; i != ifaces + n_ifaces; i++) {
 		if (i->fd_tcp_count == 0) { // Ignore XDP interface.
 			assert(i->fd_xdp_count > 0);
@@ -150,10 +147,13 @@ static unsigned tcp_set_ifaces(const iface_t *ifaces, size_t n_ifaces,
 			tcp_id = thread_id - i->fd_udp_count;
 		}
 #endif
-		fdset_add(fds, i->fd_tcp[tcp_id], POLLIN, NULL);
+		int ret = fdset_add(fds, i->fd_tcp[tcp_id], FDSET_POLLIN, NULL);
+		if (ret < 0) {
+			return 0;
+		}
 	}
 
-	return fds->n;
+	return fdset_get_length(fds);
 }
 
 static int tcp_handle(tcp_context_t *tcp, int fd, struct iovec *rx, struct iovec *tx)
@@ -205,7 +205,7 @@ static int tcp_handle(tcp_context_t *tcp, int fd, struct iovec *rx, struct iovec
 		/* Send, if response generation passed and wasn't ignored. */
 		if (ans->size > 0 && tcp_send_state(tcp->layer.state)) {
 			int sent = net_dns_tcp_send(fd, ans->wire, ans->size,
-			                            tcp->io_timeout);
+			                            tcp->io_timeout, NULL);
 			if (sent != ans->size) {
 				tcp_log_error(&ss, "send", sent);
 				ret = KNOT_EOF;
@@ -226,28 +226,28 @@ static int tcp_handle(tcp_context_t *tcp, int fd, struct iovec *rx, struct iovec
 static void tcp_event_accept(tcp_context_t *tcp, unsigned i)
 {
 	/* Accept client. */
-	int fd = tcp->set.pfd[i].fd;
+	int fd = fdset_get_fd(&tcp->set, i);
 	int client = net_accept(fd, NULL);
 	if (client >= 0) {
 		/* Assign to fdset. */
-		int next_id = fdset_add(&tcp->set, client, POLLIN, NULL);
-		if (next_id < 0) {
+		int idx = fdset_add(&tcp->set, client, FDSET_POLLIN, NULL);
+		if (idx < 0) {
 			close(client);
 			return;
 		}
 
 		/* Update watchdog timer. */
-		fdset_set_watchdog(&tcp->set, next_id, tcp->idle_timeout);
+		(void)fdset_set_watchdog(&tcp->set, idx, tcp->idle_timeout);
 	}
 }
 
 static int tcp_event_serve(tcp_context_t *tcp, unsigned i)
 {
-	int fd = tcp->set.pfd[i].fd;
-	int ret = tcp_handle(tcp, fd, &tcp->iov[0], &tcp->iov[1]);
+	int ret = tcp_handle(tcp, fdset_get_fd(&tcp->set, i),
+	                     &tcp->iov[0], &tcp->iov[1]);
 	if (ret == KNOT_EOK) {
 		/* Update socket activity timer. */
-		fdset_set_watchdog(&tcp->set, i, tcp->idle_timeout);
+		(void)fdset_set_watchdog(&tcp->set, i, tcp->idle_timeout);
 	}
 
 	return ret;
@@ -258,47 +258,45 @@ static void tcp_wait_for_events(tcp_context_t *tcp)
 	fdset_t *set = &tcp->set;
 
 	/* Check if throttled with many open TCP connections. */
-	assert(set->n <= tcp->max_worker_fds);
-	tcp->is_throttled = set->n == tcp->max_worker_fds;
+	assert(fdset_get_length(set) <= tcp->max_worker_fds);
+	tcp->is_throttled = fdset_get_length(set) == tcp->max_worker_fds;
 
 	/* If throttled, temporarily ignore new TCP connections. */
-	unsigned i = tcp->is_throttled ? tcp->client_threshold : 0;
+	unsigned offset = tcp->is_throttled ? tcp->client_threshold : 0;
 
 	/* Wait for events. */
-	int nfds = poll(&(set->pfd[i]), set->n - i, TCP_SWEEP_INTERVAL * 1000);
+	fdset_it_t it;
+	(void)fdset_poll(set, &it, offset, TCP_SWEEP_INTERVAL * 1000);
 
 	/* Mark the time of last poll call. */
 	tcp->last_poll_time = time_now();
 
 	/* Process events. */
-	while (nfds > 0 && i < set->n) {
+	for (; !fdset_it_is_done(&it); fdset_it_next(&it)) {
 		bool should_close = false;
-		if (set->pfd[i].revents & (POLLERR|POLLHUP|POLLNVAL)) {
-			should_close = (i >= tcp->client_threshold);
-			--nfds;
-		} else if (set->pfd[i].revents & (POLLIN)) {
+		unsigned int idx = fdset_it_get_idx(&it);
+		if (fdset_it_is_error(&it)) {
+			should_close = (idx >= tcp->client_threshold);
+		} else if (fdset_it_is_pollin(&it)) {
 			/* Master sockets - new connection to accept. */
-			if (i < tcp->client_threshold) {
+			if (idx < tcp->client_threshold) {
 				/* Don't accept more clients than configured. */
-				if (set->n < tcp->max_worker_fds) {
-					tcp_event_accept(tcp, i);
+				if (fdset_get_length(set) < tcp->max_worker_fds) {
+					tcp_event_accept(tcp, idx);
 				}
 			/* Client sockets - already accepted connection or
 			   closed connection :-( */
-			} else if (tcp_event_serve(tcp, i) != KNOT_EOK) {
+			} else if (tcp_event_serve(tcp, idx) != KNOT_EOK) {
 				should_close = true;
 			}
-			--nfds;
 		}
 
 		/* Evaluate. */
 		if (should_close) {
-			close(set->pfd[i].fd);
-			fdset_remove(set, i);
-		} else {
-			++i;
+			fdset_it_remove(&it);
 		}
 	}
+	fdset_it_commit(&it);
 }
 
 int tcp_master(dthread_t *thread)
@@ -335,9 +333,6 @@ int tcp_master(dthread_t *thread)
 	};
 	knot_layer_init(&tcp.layer, &mm, process_query_layer());
 
-	/* Prepare initial buffer for listening and bound sockets. */
-	fdset_init(&tcp.set, FDSET_INIT_SIZE);
-
 	/* Create iovec abstraction. */
 	for (unsigned i = 0; i < 2; ++i) {
 		tcp.iov[i].iov_len = KNOT_WIRE_MAX_PKTSIZE;
@@ -353,6 +348,11 @@ int tcp_master(dthread_t *thread)
 	update_sweep_timer(&next_sweep);
 	update_tcp_conf(&tcp);
 
+	/* Prepare initial buffer for listening and bound sockets. */
+	if (fdset_init(&tcp.set, FDSET_RESIZE_STEP) != KNOT_EOK) {
+		goto finish;
+	}
+
 	/* Set descriptors for the configured interfaces. */
 	tcp.client_threshold = tcp_set_ifaces(handler->server->ifaces,
 	                                      handler->server->n_ifaces,
