@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,12 +17,12 @@
 #pragma once
 
 #include "contrib/semaphore.h"
+#include "knot/catalog/catalog_update.h"
 #include "knot/conf/conf.h"
 #include "knot/conf/confio.h"
 #include "knot/journal/journal_basic.h"
 #include "knot/events/events.h"
 #include "knot/updates/changesets.h"
-#include "knot/zone/catalog.h"
 #include "knot/zone/contents.h"
 #include "knot/zone/timers.h"
 #include "libknot/dname.h"
@@ -78,7 +78,7 @@ typedef struct zone
 	/*! \brief Control update context. */
 	struct zone_update *control_update;
 
-	/*! \brief Ensue one COW tramsaction on zone's trees at a time. */
+	/*! \brief Ensue one COW transaction on zone's trees at a time. */
 	knot_sem_t cow_lock;
 
 	/*! \brief Ptr to journal DB (in struct server) */
@@ -94,6 +94,11 @@ typedef struct zone
 	catalog_t *catalog;
 	catalog_update_t *catalog_upd;
 
+	/*! \brief Catalog-generate feature. */
+	knot_dname_t *catalog_gen;
+	catalog_update_t *cat_members;
+	const char *catalog_group;
+
 	/*! \brief Preferred master lock. Also used for flags access. */
 	pthread_mutex_t preferred_lock;
 	/*! \brief Preferred master for remote operation. */
@@ -137,9 +142,12 @@ void zone_reset(conf_t *conf, zone_t *zone);
  */
 void zone_control_clear(zone_t *zone);
 
+/*!
+ * \brief Only for RO journal operations.
+ */
 inline static zone_journal_t zone_journal(zone_t *zone)
 {
-	zone_journal_t j = { zone->journaldb, zone->name };
+	zone_journal_t j = { zone->journaldb, zone->name, NULL };
 	return j;
 }
 
