@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -56,6 +56,7 @@ enum knot_error {
 	KNOT_EEXIST        = -EEXIST,
 	KNOT_ERANGE        = -ERANGE,
 	KNOT_EADDRNOTAVAIL = -EADDRNOTAVAIL,
+	KNOT_ENOTDIR       = -ENOTDIR,
 
 	KNOT_ERRNO_ERROR   = -500,
 
@@ -98,6 +99,8 @@ enum knot_error {
 	KNOT_ELOOP,
 	KNOT_EPROGRAM,
 	KNOT_EFD,
+	KNOT_ENOPARAM,
+	KNOT_EXPARAM,
 
 	KNOT_GENERAL_ERROR = -900,
 
