@@ -42,96 +42,94 @@ static const char _yparser_actions[] = {
 };
 
 static const unsigned char _yparser_key_offsets[] = {
-	0, 0, 15, 17, 29, 31, 34, 46,
-	47, 58, 70, 72, 73, 83, 94, 98,
-	100, 103, 107, 110, 113, 126, 136, 145,
-	148, 151, 154, 158, 161, 164, 179, 194,
-	209
+	0, 0, 14, 16, 27, 29, 32, 43,
+	44, 54, 65, 67, 68, 78, 89, 93,
+	95, 98, 102, 105, 108, 121, 131, 140,
+	143, 146, 149, 153, 156, 159, 173, 187,
+	201
 };
 
 static const char _yparser_trans_keys[] = {
-	10, 13, 32, 35, 42, 45, 46, 92,
-	95, 48, 57, 65, 90, 97, 122, 10,
-	13, 32, 58, 92, 95, 45, 46, 48,
-	57, 65, 90, 97, 122, 32, 58, 10,
-	13, 32, 32, 58, 92, 95, 45, 46,
-	48, 57, 65, 90, 97, 122, 32, 32,
-	42, 46, 92, 95, 48, 57, 65, 90,
-	97, 122, 32, 58, 92, 95, 45, 46,
-	48, 57, 65, 90, 97, 122, 32, 58,
-	32, 32, 33, 34, 92, 36, 43, 45,
-	90, 94, 126, 10, 13, 32, 33, 92,
+	10, 13, 32, 35, 42, 45, 92, 95,
+	46, 57, 65, 90, 97, 122, 10, 13,
+	32, 42, 58, 92, 95, 45, 57, 65,
+	90, 97, 122, 32, 58, 10, 13, 32,
+	32, 42, 58, 92, 95, 45, 57, 65,
+	90, 97, 122, 32, 32, 42, 92, 95,
+	46, 57, 65, 90, 97, 122, 32, 42,
+	58, 92, 95, 45, 57, 65, 90, 97,
+	122, 32, 58, 32, 32, 33, 34, 92,
 	36, 43, 45, 90, 94, 126, 10, 13,
-	32, 35, 10, 13, 34, 32, 126, 34,
-	92, 32, 126, 10, 13, 32, 34, 32,
-	126, 10, 13, 32, 34, 35, 91, 92,
-	33, 43, 45, 90, 94, 126, 32, 33,
-	34, 92, 36, 43, 45, 90, 94, 126,
-	32, 33, 44, 92, 93, 36, 90, 94,
-	126, 32, 44, 93, 10, 13, 32, 34,
-	32, 126, 34, 92, 32, 126, 32, 44,
-	93, 34, 32, 126, 10, 13, 32, 35,
-	42, 45, 46, 92, 95, 48, 57, 65,
+	32, 33, 92, 36, 43, 45, 90, 94,
+	126, 10, 13, 32, 35, 10, 13, 34,
+	32, 126, 34, 92, 32, 126, 10, 13,
+	32, 34, 32, 126, 10, 13, 32, 34,
+	35, 91, 92, 33, 43, 45, 90, 94,
+	126, 32, 33, 34, 92, 36, 43, 45,
+	90, 94, 126, 32, 33, 44, 92, 93,
+	36, 90, 94, 126, 32, 44, 93, 10,
+	13, 32, 34, 32, 126, 34, 92, 32,
+	126, 32, 44, 93, 34, 32, 126, 10,
+	13, 32, 35, 42, 45, 92, 95, 46,
+	57, 65, 90, 97, 122, 10, 13, 32,
+	35, 42, 45, 92, 95, 46, 57, 65,
 	90, 97, 122, 10, 13, 32, 35, 42,
-	45, 46, 92, 95, 48, 57, 65, 90,
-	97, 122, 10, 13, 32, 35, 42, 45,
-	46, 92, 95, 48, 57, 65, 90, 97,
-	122, 10, 13, 32, 35, 42, 45, 46,
-	92, 95, 48, 57, 65, 90, 97, 122,
-	0
+	45, 92, 95, 46, 57, 65, 90, 97,
+	122, 10, 13, 32, 35, 42, 45, 92,
+	95, 46, 57, 65, 90, 97, 122, 0
 };
 
 static const char _yparser_single_lengths[] = {
-	0, 9, 2, 4, 2, 3, 4, 1,
-	5, 4, 2, 1, 4, 5, 4, 2,
+	0, 8, 2, 5, 2, 3, 5, 1,
+	4, 5, 2, 1, 4, 5, 4, 2,
 	1, 2, 3, 1, 7, 4, 5, 3,
-	3, 1, 2, 3, 1, 9, 9, 9,
-	9
+	3, 1, 2, 3, 1, 8, 8, 8,
+	8
 };
 
 static const char _yparser_range_lengths[] = {
-	0, 3, 0, 4, 0, 0, 4, 0,
-	3, 4, 0, 0, 3, 3, 0, 0,
+	0, 3, 0, 3, 0, 0, 3, 0,
+	3, 3, 0, 0, 3, 3, 0, 0,
 	1, 1, 0, 1, 3, 3, 2, 0,
 	0, 1, 1, 0, 1, 3, 3, 3,
 	3
 };
 
 static const unsigned char _yparser_index_offsets[] = {
-	0, 0, 13, 16, 25, 28, 32, 41,
-	43, 52, 61, 64, 66, 74, 83, 88,
-	91, 94, 98, 102, 105, 116, 124, 132,
-	136, 140, 143, 147, 151, 154, 167, 180,
-	193
+	0, 0, 12, 15, 24, 27, 31, 40,
+	42, 50, 59, 62, 64, 72, 81, 86,
+	89, 92, 96, 100, 103, 114, 122, 130,
+	134, 138, 141, 145, 149, 152, 164, 176,
+	188
 };
 
 static const char _yparser_indicies[] = {
 	1, 2, 3, 4, 5, 6, 5, 5,
-	5, 5, 5, 5, 0, 1, 2, 4,
-	7, 9, 8, 8, 8, 8, 8, 8,
-	0, 10, 11, 0, 12, 13, 14, 0,
-	15, 17, 16, 16, 16, 16, 16, 16,
-	0, 18, 0, 18, 19, 19, 19, 19,
-	19, 19, 19, 0, 20, 22, 21, 21,
-	21, 21, 21, 21, 0, 23, 24, 0,
-	25, 0, 25, 26, 27, 28, 26, 26,
-	26, 0, 29, 30, 31, 32, 33, 32,
-	32, 32, 0, 12, 13, 34, 35, 0,
-	12, 13, 35, 36, 32, 0, 38, 39,
-	37, 0, 29, 30, 31, 0, 40, 37,
-	0, 12, 13, 14, 27, 35, 41, 28,
-	26, 26, 26, 0, 41, 42, 43, 44,
-	42, 42, 42, 0, 45, 46, 47, 48,
-	49, 46, 46, 0, 50, 41, 51, 0,
-	12, 13, 34, 0, 52, 46, 0, 54,
-	55, 53, 0, 45, 47, 49, 0, 56,
-	53, 0, 1, 2, 3, 4, 57, 6,
-	57, 57, 57, 57, 57, 57, 0, 59,
-	60, 61, 62, 63, 64, 63, 63, 63,
-	63, 63, 63, 58, 65, 66, 67, 68,
-	69, 70, 69, 69, 69, 69, 69, 69,
-	0, 71, 72, 73, 74, 75, 76, 75,
-	75, 75, 75, 75, 75, 58, 0
+	5, 5, 5, 0, 1, 2, 4, 7,
+	8, 9, 8, 8, 8, 8, 8, 0,
+	10, 11, 0, 12, 13, 14, 0, 15,
+	16, 17, 16, 16, 16, 16, 16, 0,
+	18, 0, 18, 19, 19, 19, 19, 19,
+	19, 0, 20, 21, 22, 21, 21, 21,
+	21, 21, 0, 23, 24, 0, 25, 0,
+	25, 26, 27, 28, 26, 26, 26, 0,
+	29, 30, 31, 32, 33, 32, 32, 32,
+	0, 12, 13, 34, 35, 0, 12, 13,
+	35, 36, 32, 0, 38, 39, 37, 0,
+	29, 30, 31, 0, 40, 37, 0, 12,
+	13, 14, 27, 35, 41, 28, 26, 26,
+	26, 0, 41, 42, 43, 44, 42, 42,
+	42, 0, 45, 46, 47, 48, 49, 46,
+	46, 0, 50, 41, 51, 0, 12, 13,
+	34, 0, 52, 46, 0, 54, 55, 53,
+	0, 45, 47, 49, 0, 56, 53, 0,
+	1, 2, 3, 4, 57, 6, 57, 57,
+	57, 57, 57, 0, 59, 60, 61, 62,
+	63, 64, 63, 63, 63, 63, 63, 58,
+	65, 66, 67, 68, 69, 70, 69, 69,
+	69, 69, 69, 0, 71, 72, 73, 74,
+	75, 76, 75, 75, 75, 75, 75, 58,
+	0
 };
 
 static const char _yparser_trans_targs[] = {
