@@ -25,6 +25,7 @@
 #include "contrib/strtonum.h"
 #include "knot/conf/conf.h"
 #include "knot/dnssec/zone-keys.h"
+#include "libknot/attribute.h"
 #include "libknot/libknot.h"
 #include "utils/common/params.h"
 #include "utils/keymgr/functions.h"
@@ -341,7 +342,7 @@ static void update_privileges(void)
 	}
 
 	// Just try to alter process privileges if different from configured.
-	int unused __attribute__((unused));
+	_unused_ int unused;
 	if ((gid_t)gid != getgid()) {
 		unused = setregid(gid, gid);
 	}
@@ -436,7 +437,7 @@ int main(int argc, char *argv[])
 	update_privileges();
 
 	knot_lmdb_db_t kaspdb = { 0 };
-	conf_val_t mapsize = conf_db_param(conf(), C_KASP_DB_MAX_SIZE, C_MAX_KASP_DB_SIZE);
+	conf_val_t mapsize = conf_db_param(conf(), C_KASP_DB_MAX_SIZE);
 	char *kasp_dir = conf_db(conf(), C_KASP_DB);
 	knot_lmdb_init(&kaspdb, kasp_dir, conf_int(&mapsize), 0, "keys_db");
 	free(kasp_dir);
